@@ -1,6 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- * Implmentation of DAAP (e.g., iTunes Music) sharing
+/* Implmentation of DAAP (e.g., iTunes Music) sharing
  *
  * Copyright (C) 2005 Charles Schmidt <cschmidt2@emich.edu>
  *
@@ -38,75 +36,63 @@
 
 #include <libdmapsharing/dmap.h>
 #include <libdmapsharing/dmap-structure.h>
+#include <libdmapsharing/dmap-utils.h>
 
 #ifdef HAVE_GSTREAMERAPP
-#include <libdmapsharing/g-gst-mp3-input-stream.h>
-#include <libdmapsharing/g-gst-wav-input-stream.h>
+#include <libdmapsharing/dmap-gst-input-stream.h>
 #endif /* HAVE_GSTREAMERAPP */
 
-static void daap_share_set_property  (GObject *object,
-					 guint prop_id,
-					 const GValue *value,
-					 GParamSpec *pspec);
-static void daap_share_get_property  (GObject *object,
-					 guint prop_id,
-					 GValue *value,
-				 	 GParamSpec *pspec);
-static void daap_share_dispose	(GObject *object);
-guint daap_share_get_desired_port (DMAPShare *share);
-const char *daap_share_get_type_of_service (DMAPShare *share);
-void daap_share_server_info (DMAPShare         *share,
-			     SoupServer        *server,
-	  		     SoupMessage       *message,
-			     const char        *path,
-			     GHashTable        *query,
-			     SoupClientContext *context);
-void daap_share_message_add_standard_headers (DMAPShare *share,
-					      SoupMessage *message);
-static void databases_browse_xxx (DMAPShare *share,
-				  SoupServer *server,
-				  SoupMessage *msg,
+static void daap_share_set_property (GObject * object,
+				     guint prop_id,
+				     const GValue * value,
+				     GParamSpec * pspec);
+static void daap_share_get_property (GObject * object,
+				     guint prop_id,
+				     GValue * value, GParamSpec * pspec);
+static void daap_share_dispose (GObject * object);
+guint daap_share_get_desired_port (DMAPShare * share);
+const char *daap_share_get_type_of_service (DMAPShare * share);
+void daap_share_server_info (DMAPShare * share,
+			     SoupServer * server,
+			     SoupMessage * message,
+			     const char *path,
+			     GHashTable * query, SoupClientContext * context);
+void daap_share_message_add_standard_headers (DMAPShare * share,
+					      SoupMessage * message);
+static void databases_browse_xxx (DMAPShare * share,
+				  SoupServer * server,
+				  SoupMessage * msg,
 				  const char *path,
-				  GHashTable *query,
-				  SoupClientContext *context);
-static void databases_items_xxx (DMAPShare *share,
-				 SoupServer *server,
-				 SoupMessage *msg,
+				  GHashTable * query,
+				  SoupClientContext * context);
+static void databases_items_xxx (DMAPShare * share,
+				 SoupServer * server,
+				 SoupMessage * msg,
 				 const char *path,
-				 GHashTable *query,
-				 SoupClientContext *context);
-static struct DMAPMetaDataMap *get_meta_data_map (DMAPShare *share);
-static void add_entry_to_mlcl (gpointer id,
-			       DMAPRecord *record,
-			       gpointer mb);
+				 GHashTable * query,
+				 SoupClientContext * context);
+static struct DMAPMetaDataMap *get_meta_data_map (DMAPShare * share);
+static void add_entry_to_mlcl (gpointer id, DMAPRecord * record, gpointer mb);
 
 #define DAAP_TYPE_OF_SERVICE "_daap._tcp"
 #define DAAP_PORT 3689
 
-struct DAAPSharePrivate {
+struct DAAPSharePrivate
+{
 	gchar unused;
 };
 
-#define DAAP_SHARE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DAAP_SHARE, DAAPSharePrivate))
+#define DAAP_SHARE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DAAP_TYPE_SHARE, DAAPSharePrivate))
 
-enum {
+enum
+{
 	PROP_0,
 };
 
-/* Provide two items as user_data to callback. */
-typedef struct ChunkData {
-	SoupServer *server;
-	GInputStream *stream;
-} ChunkData;
-
-
-G_DEFINE_TYPE (DAAPShare, daap_share, TYPE_DMAP_SHARE)
-
-/* FIXME: get rid of this global: */
-static gchar *transcode_format = NULL;
+G_DEFINE_TYPE (DAAPShare, daap_share, DMAP_TYPE_SHARE);
 
 static void
-daap_share_class_init (DAAPShareClass *klass)
+daap_share_class_init (DAAPShareClass * klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	DMAPShareClass *parent_class = DMAP_SHARE_CLASS (object_class);
@@ -115,30 +101,30 @@ daap_share_class_init (DAAPShareClass *klass)
 	object_class->set_property = daap_share_set_property;
 	object_class->dispose = daap_share_dispose;
 
-	parent_class->get_desired_port     = daap_share_get_desired_port;
-	parent_class->get_type_of_service  = daap_share_get_type_of_service;
-	parent_class->message_add_standard_headers = daap_share_message_add_standard_headers;
-	parent_class->get_meta_data_map    = get_meta_data_map;
-	parent_class->add_entry_to_mlcl    = add_entry_to_mlcl;
+	parent_class->get_desired_port = daap_share_get_desired_port;
+	parent_class->get_type_of_service = daap_share_get_type_of_service;
+	parent_class->message_add_standard_headers =
+		daap_share_message_add_standard_headers;
+	parent_class->get_meta_data_map = get_meta_data_map;
+	parent_class->add_entry_to_mlcl = add_entry_to_mlcl;
 	parent_class->databases_browse_xxx = databases_browse_xxx;
-	parent_class->databases_items_xxx  = databases_items_xxx;
-	parent_class->server_info          = daap_share_server_info;
+	parent_class->databases_items_xxx = databases_items_xxx;
+	parent_class->server_info = daap_share_server_info;
 
 	g_type_class_add_private (klass, sizeof (DAAPSharePrivate));
 }
 
 static void
-daap_share_init (DAAPShare *share)
+daap_share_init (DAAPShare * share)
 {
 	share->priv = DAAP_SHARE_GET_PRIVATE (share);
 	/* FIXME: do I need to manually call parent _init? */
 }
 
 static void
-daap_share_set_property (GObject *object,
-			    guint prop_id,
-			    const GValue *value,
-			    GParamSpec *pspec)
+daap_share_set_property (GObject * object,
+			 guint prop_id,
+			 const GValue * value, GParamSpec * pspec)
 {
 	// DAAPShare *share = DAAP_SHARE (object);
 
@@ -150,10 +136,8 @@ daap_share_set_property (GObject *object,
 }
 
 static void
-daap_share_get_property (GObject *object,
-			    guint prop_id,
-			    GValue *value,
-			    GParamSpec *pspec)
+daap_share_get_property (GObject * object,
+			 guint prop_id, GValue * value, GParamSpec * pspec)
 {
 	// DAAPShare *share = DAAP_SHARE (object);
 
@@ -165,18 +149,19 @@ daap_share_get_property (GObject *object,
 }
 
 static void
-daap_share_dispose (GObject *object)
+daap_share_dispose (GObject * object)
 {
 	/* FIXME: implement in parent */
 }
 
-static gchar *mime_to_format (const gchar *transcode_mimetype)
+static gchar *
+mime_to_format (const gchar * transcode_mimetype)
 {
-	if (! transcode_mimetype) {
+	if (!transcode_mimetype) {
 		return NULL;
-	} else if (! strcmp (transcode_mimetype, "audio/wav")) {
+	} else if (!strcmp (transcode_mimetype, "audio/wav")) {
 		return g_strdup ("wav");
-	} else if ( ! strcmp (transcode_mimetype, "audio/mp3")) {
+	} else if (!strcmp (transcode_mimetype, "audio/mp3")) {
 		return g_strdup ("mp3");
 	} else
 		return NULL;
@@ -185,35 +170,34 @@ static gchar *mime_to_format (const gchar *transcode_mimetype)
 DAAPShare *
 daap_share_new (const char *name,
 		const char *password,
-		DMAPDb *db,
-		DMAPContainerDb *container_db,
-		gchar *transcode_mimetype)
+		DMAPDb * db,
+		DMAPContainerDb * container_db, gchar * transcode_mimetype)
 {
 	DAAPShare *share;
 
 	g_object_ref (db);
 	g_object_ref (container_db);
 
-	share = DAAP_SHARE (g_object_new (TYPE_DAAP_SHARE,
-					     "name", name,
-					     "password", password,
-					     "db", db,
-					     "container-db", container_db,
-					     "transcode-mimetype", transcode_mimetype,
-					     NULL));
+	share = DAAP_SHARE (g_object_new (DAAP_TYPE_SHARE,
+					  "name", name,
+					  "password", password,
+					  "db", db,
+					  "container-db", container_db,
+					  "transcode-mimetype",
+					  transcode_mimetype, NULL));
 
 	_dmap_share_server_start (DMAP_SHARE (share));
 	_dmap_share_publish_start (DMAP_SHARE (share));
 
-	transcode_format = mime_to_format (transcode_mimetype);
-
 	return share;
 }
 
 void
-daap_share_message_add_standard_headers (DMAPShare *share, SoupMessage *message)
+daap_share_message_add_standard_headers (DMAPShare * share,
+					 SoupMessage * message)
 {
-	soup_message_headers_append (message->response_headers, "DMAP-Server", "libdmapsharing" VERSION);
+	soup_message_headers_append (message->response_headers, "DMAP-Server",
+				     "libdmapsharing" VERSION);
 }
 
 #define DMAP_VERSION 2.0
@@ -221,24 +205,23 @@ daap_share_message_add_standard_headers (DMAPShare *share, SoupMessage *message)
 #define DAAP_TIMEOUT 1800
 
 guint
-daap_share_get_desired_port (DMAPShare *share)
+daap_share_get_desired_port (DMAPShare * share)
 {
 	return DAAP_PORT;
 }
 
 const char *
-daap_share_get_type_of_service (DMAPShare *share)
+daap_share_get_type_of_service (DMAPShare * share)
 {
 	return DAAP_TYPE_OF_SERVICE;
 }
 
 void
-daap_share_server_info (DMAPShare *share,
-		SoupServer        *server,
-	  	SoupMessage       *message,
-		const char        *path,
-		GHashTable        *query,
-		SoupClientContext *context)
+daap_share_server_info (DMAPShare * share,
+			SoupServer * server,
+			SoupMessage * message,
+			const char *path,
+			GHashTable * query, SoupClientContext * context)
 {
 /* MSRV	server info response
  * 	MSTT status
@@ -278,7 +261,8 @@ daap_share_server_info (DMAPShare *share,
 	 * 3.0 is 2/3
 	 */
 	dmap_structure_add (msrv, DMAP_CC_MINM, nameprop);
-	dmap_structure_add (msrv, DMAP_CC_MSAU, _dmap_share_get_auth_method (share));
+	dmap_structure_add (msrv, DMAP_CC_MSAU,
+			    _dmap_share_get_auth_method (share));
 	/* authentication method
 	 * 0 is nothing
 	 * 1 is name & password
@@ -302,7 +286,8 @@ daap_share_server_info (DMAPShare *share,
 	g_free (nameprop);
 }
 
-typedef enum {
+typedef enum
+{
 	ITEM_ID = 0,
 	ITEM_NAME,
 	ITEM_KIND,
@@ -350,103 +335,69 @@ typedef enum {
 } DAAPMetaData;
 
 static struct DMAPMetaDataMap meta_data_map[] = {
-	{"dmap.itemid",			ITEM_ID},
-    	{"dmap.itemname",		ITEM_NAME},
-    	{"dmap.itemkind",		ITEM_KIND},
-    	{"dmap.persistentid",		PERSISTENT_ID},
-	{"dmap.containeritemid",	CONTAINER_ITEM_ID},
-    	{"daap.songalbum",		SONG_ALBUM},
-    	{"daap.songartist",		SONG_ARTIST},
-    	{"daap.songbitrate",		SONG_BITRATE},
-    	{"daap.songbeatsperminute",	SONG_BPM},
-    	{"daap.songcomment",		SONG_COMMENT},
-    	{"daap.songcompilation",	SONG_COMPILATION},
-    	{"daap.songcomposer",		SONG_COMPOSER},
-    	{"daap.songdatakind",		SONG_DATA_KIND},
-    	{"daap.songdataurl",		SONG_DATA_URL},
-    	{"daap.songdateadded",		SONG_DATE_ADDED},
-    	{"daap.songdatemodified",	SONG_DATE_MODIFIED},
-    	{"daap.songdescription",	SONG_DESCRIPTION},
-    	{"daap.songdisabled",		SONG_DISABLED},
-    	{"daap.songdisccount",		SONG_DISC_COUNT},
-    	{"daap.songdiscnumber",		SONG_DISC_NUMBER},
-    	{"daap.songeqpreset",		SONG_EQ_PRESET},
-    	{"daap.songformat",		SONG_FORMAT},
-    	{"daap.songgenre",		SONG_GENRE},
-    	{"daap.songgrouping",		SONG_GROUPING},
-    	{"daap.songrelativevolume",	SONG_RELATIVE_VOLUME},
-    	{"daap.songsamplerate",		SONG_SAMPLE_RATE},
-    	{"daap.songsize",		SONG_SIZE},
-    	{"daap.songstarttime",		SONG_START_TIME},
-    	{"daap.songstoptime",		SONG_STOP_TIME},
-   	{"daap.songtime",		SONG_TIME},
-    	{"daap.songtrackcount",		SONG_TRACK_COUNT},
-    	{"daap.songtracknumber",	SONG_TRACK_NUMBER},
-    	{"daap.songuserrating",		SONG_USER_RATING},
-    	{"daap.songyear",		SONG_YEAR},
-	{"daap.sortalbum",		SONG_SORT_ALBUM},
-	{"daap.sortartist",		SONG_SORT_ARTIST},
-	{"com.apple.itunes.has-video",	SONG_HAS_VIDEO},
-	{"com.apple.itunes.smart-playlist",		SONG_SMART_PLAYLIST},
-	{"com.apple.itunes.is-podcast-playlist",	SONG_IS_PODCAST_PLAYLIST},
-	{"com.apple.itunes.special-playlist",		SONG_SPECIAL_PLAYLIST},
-	{"com.apple.itunes.saved-genius",		SONG_SAVED_GENIUS},
-	{"com.apple.itunes.mediakind",			SONG_MEDIAKIND},
-	{"dmap.haschildcontainers",			HAS_CHILD_CONTAINERS},
-	{"dmap.parentcontainerid",			PARENT_CONTAINER_ID},
-	{ NULL,				0}};
+	{"dmap.itemid", ITEM_ID},
+	{"dmap.itemname", ITEM_NAME},
+	{"dmap.itemkind", ITEM_KIND},
+	{"dmap.persistentid", PERSISTENT_ID},
+	{"dmap.containeritemid", CONTAINER_ITEM_ID},
+	{"daap.songalbum", SONG_ALBUM},
+	{"daap.songartist", SONG_ARTIST},
+	{"daap.songbitrate", SONG_BITRATE},
+	{"daap.songbeatsperminute", SONG_BPM},
+	{"daap.songcomment", SONG_COMMENT},
+	{"daap.songcompilation", SONG_COMPILATION},
+	{"daap.songcomposer", SONG_COMPOSER},
+	{"daap.songdatakind", SONG_DATA_KIND},
+	{"daap.songdataurl", SONG_DATA_URL},
+	{"daap.songdateadded", SONG_DATE_ADDED},
+	{"daap.songdatemodified", SONG_DATE_MODIFIED},
+	{"daap.songdescription", SONG_DESCRIPTION},
+	{"daap.songdisabled", SONG_DISABLED},
+	{"daap.songdisccount", SONG_DISC_COUNT},
+	{"daap.songdiscnumber", SONG_DISC_NUMBER},
+	{"daap.songeqpreset", SONG_EQ_PRESET},
+	{"daap.songformat", SONG_FORMAT},
+	{"daap.songgenre", SONG_GENRE},
+	{"daap.songgrouping", SONG_GROUPING},
+	{"daap.songrelativevolume", SONG_RELATIVE_VOLUME},
+	{"daap.songsamplerate", SONG_SAMPLE_RATE},
+	{"daap.songsize", SONG_SIZE},
+	{"daap.songstarttime", SONG_START_TIME},
+	{"daap.songstoptime", SONG_STOP_TIME},
+	{"daap.songtime", SONG_TIME},
+	{"daap.songtrackcount", SONG_TRACK_COUNT},
+	{"daap.songtracknumber", SONG_TRACK_NUMBER},
+	{"daap.songuserrating", SONG_USER_RATING},
+	{"daap.songyear", SONG_YEAR},
+	{"daap.sortalbum", SONG_SORT_ALBUM},
+	{"daap.sortartist", SONG_SORT_ARTIST},
+	{"com.apple.itunes.has-video", SONG_HAS_VIDEO},
+	{"com.apple.itunes.smart-playlist", SONG_SMART_PLAYLIST},
+	{"com.apple.itunes.is-podcast-playlist", SONG_IS_PODCAST_PLAYLIST},
+	{"com.apple.itunes.special-playlist", SONG_SPECIAL_PLAYLIST},
+	{"com.apple.itunes.saved-genius", SONG_SAVED_GENIUS},
+	{"com.apple.itunes.mediakind", SONG_MEDIAKIND},
+	{"dmap.haschildcontainers", HAS_CHILD_CONTAINERS},
+	{"dmap.parentcontainerid", PARENT_CONTAINER_ID},
+	{NULL, 0}
+};
 
 #define DAAP_ITEM_KIND_AUDIO 2
 #define DAAP_SONG_DATA_KIND_NONE 0
 
 static void
-write_next_chunk (SoupMessage *message, ChunkData *cd)
-{
-	gssize read_size;
-	GError *error = NULL;
-	gchar *chunk = g_malloc (DMAP_SHARE_CHUNK_SIZE);
-
-	read_size = g_input_stream_read (cd->stream,
-					 chunk,
-					 DMAP_SHARE_CHUNK_SIZE,
-					 NULL,
-					 &error);
-	if (read_size > 0) {
-		soup_message_body_append (message->response_body,
-					  SOUP_MEMORY_TAKE,
-					  chunk,
-					  read_size);
-	} else {
-		if (error != NULL) {
-			g_warning ("Error reading from input stream: %s",
-				   error->message);
-			g_error_free (error);
-		}
-		g_free (chunk);
-		g_debug ("Wrote 0 bytes, sending message complete.");
-		soup_message_body_complete (message->response_body);
-	}
-	soup_server_unpause_message (cd->server, message);
-}
-
-static void
-chunked_message_finished (SoupMessage *message, ChunkData *cd)
-{
-	g_debug ("Finished sending chunked file.");
-	g_input_stream_close (cd->stream, NULL, NULL);
-	g_free (cd);
-}
-
-static void
-send_chunked_file (SoupServer *server, SoupMessage *message, DAAPRecord *record, guint64 filesize, guint64 offset, const gchar *transcode_mimetype)
+send_chunked_file (SoupServer * server, SoupMessage * message,
+		   DAAPRecord * record, guint64 filesize, guint64 offset,
+		   const gchar * transcode_mimetype)
 {
+	gchar *format = NULL;
 	GInputStream *stream;
 	gboolean has_video;
 	const char *location;
 	GError *error = NULL;
 	ChunkData *cd = g_new (ChunkData, 1);
 
-	g_object_get (record, "location", &location, NULL);
+	g_object_get (record, "location", &location, "has-video", &has_video, NULL);
 
 	/* FIXME: This crashes on powerpc-440fp-linux-gnu:
 	 * g_debug ("Sending %s chunked from offset %" G_GUINT64_FORMAT ".", location, offset);
@@ -459,23 +410,31 @@ send_chunked_file (SoupServer *server, SoupMessage *message, DAAPRecord *record,
 	if (error != NULL) {
 		g_warning ("Couldn't open %s: %s.", location, error->message);
 		g_error_free (error);
-		soup_message_set_status (message, SOUP_STATUS_INTERNAL_SERVER_ERROR);
+		soup_message_set_status (message,
+					 SOUP_STATUS_INTERNAL_SERVER_ERROR);
 		g_free (cd);
 		return;
 	}
 
-	if (transcode_format == NULL) {
+	g_object_get (record, "format", &format, NULL);
+	// Not presently transcoding videos (see also same comments elsewhere).
+	if (has_video || transcode_mimetype == NULL
+	    || !strcmp (format, mime_to_format (transcode_mimetype))) {
+		g_debug ("Not transcoding");
 		cd->stream = stream;
 #ifdef HAVE_GSTREAMERAPP
-	} else if (! strcmp ("mp3", transcode_format)) {
-		cd->stream = G_INPUT_STREAM (g_gst_mp3_input_stream_new (stream));
-	} else if (! strcmp ("wav", transcode_format)) {
-		cd->stream = G_INPUT_STREAM (g_gst_wav_input_stream_new (stream));
-#endif /* HAVE_GSTREAMERAPP */
 	} else {
-		g_warning ("Transcode format %s not supported", transcode_format);
+		cd->stream =
+			dmap_gst_input_stream_new (transcode_mimetype,
+						   stream);
+	}
+#else
+	} else {
+		g_warning ("Transcode format %s not supported",
+			   transcode_mimetype);
 		cd->stream = stream;
 	}
+#endif /* HAVE_GSTREAMERAPP */
 
 	if (cd->stream == NULL) {
 		g_warning ("Could not set up input stream");
@@ -484,35 +443,39 @@ send_chunked_file (SoupServer *server, SoupMessage *message, DAAPRecord *record,
 	}
 
 	if (offset != 0) {
-		if (g_seekable_seek (G_SEEKABLE (cd->stream), offset, G_SEEK_SET, NULL, &error) == FALSE) {
+		if (g_seekable_seek
+		    (G_SEEKABLE (cd->stream), offset, G_SEEK_SET, NULL,
+		     &error) == FALSE) {
 			g_warning ("Error seeking: %s.", error->message);
 			g_input_stream_close (cd->stream, NULL, NULL);
-			soup_message_set_status (message, SOUP_STATUS_INTERNAL_SERVER_ERROR);
+			soup_message_set_status (message,
+						 SOUP_STATUS_INTERNAL_SERVER_ERROR);
 			g_free (cd);
 			return;
-	 	}
+		}
 		filesize -= offset;
 	}
 
 	/* Free memory after each chunk sent out over network. */
 	soup_message_body_set_accumulate (message->response_body, FALSE);
 
-	g_object_get (record, "has-video", &has_video, NULL);
+	// Not presently transcoding videos (see also same comments elsewhere).
 	if (has_video
-		/* NOTE: iTunes seems to require this or it stops reading 
-		 * video data after about 2.5MB. Perhaps this is so iTunes
-		 * knows how much data to buffer.
-		 */
-	    || transcode_format == NULL) {
-	    	/* NOTE: iTunes 8 (and other versions?) will not seek
+	    /* NOTE: iTunes seems to require this or it stops reading 
+	     * video data after about 2.5MB. Perhaps this is so iTunes
+	     * knows how much data to buffer.
+	     */
+	    || transcode_mimetype == NULL) {
+		/* NOTE: iTunes 8 (and other versions?) will not seek
 		 * properly without a Content-Length header.
 		 */
 		g_debug ("Using HTTP 1.1 content length encoding.");
 		soup_message_headers_set_encoding (message->response_headers,
-						  SOUP_ENCODING_CONTENT_LENGTH);
-		g_debug ("Content length is %" G_GUINT64_FORMAT ".", filesize);
-		soup_message_headers_set_content_length (
-			message->response_headers, filesize);
+						   SOUP_ENCODING_CONTENT_LENGTH);
+		g_debug ("Content length is %" G_GUINT64_FORMAT ".",
+			 filesize);
+		soup_message_headers_set_content_length
+			(message->response_headers, filesize);
 	} else if (soup_message_get_http_version (message) == SOUP_HTTP_1_0) {
 		/* NOTE: Roku clients support only HTTP 1.0. */
 #ifdef HAVE_ENCODING_EOF
@@ -520,7 +483,8 @@ send_chunked_file (SoupServer *server, SoupMessage *message, DAAPRecord *record,
 		soup_message_headers_set_encoding (message->response_headers,
 						   SOUP_ENCODING_EOF);
 #else
-		g_warning ("Received HTTP 1.0 request, but not built with HTTP 1.0 support");
+		g_warning
+			("Received HTTP 1.0 request, but not built with HTTP 1.0 support");
 		soup_message_headers_set_encoding (message->response_headers,
 						   SOUP_ENCODING_CHUNKED);
 #endif
@@ -533,72 +497,89 @@ send_chunked_file (SoupServer *server, SoupMessage *message, DAAPRecord *record,
 						   SOUP_ENCODING_CHUNKED);
 	}
 
-	soup_message_headers_append (message->response_headers, "Connection", "Close");
-	soup_message_headers_append (message->response_headers, "Content-Type", "application/x-dmap-tagged");
+	soup_message_headers_append (message->response_headers, "Connection",
+				     "Close");
+	soup_message_headers_append (message->response_headers,
+				     "Content-Type",
+				     "application/x-dmap-tagged");
 
-	g_signal_connect (message, "wrote_headers", G_CALLBACK (write_next_chunk), cd);
-	g_signal_connect (message, "wrote_chunk", G_CALLBACK (write_next_chunk), cd);
-	g_signal_connect (message, "finished", G_CALLBACK (chunked_message_finished), cd);
+	g_signal_connect (message, "wrote_headers",
+			  G_CALLBACK (dmap_write_next_chunk), cd);
+	g_signal_connect (message, "wrote_chunk",
+			  G_CALLBACK (dmap_write_next_chunk), cd);
+	g_signal_connect (message, "finished",
+			  G_CALLBACK (dmap_chunked_message_finished), cd);
 	/* NOTE: cd g_free'd by chunked_message_finished(). */
 }
 
 static void
-add_entry_to_mlcl (gpointer id,
-		   DMAPRecord *record,
-		   gpointer _mb)
+add_entry_to_mlcl (gpointer id, DMAPRecord * record, gpointer _mb)
 {
 	GNode *mlit;
+	gboolean has_video = 0;
 	struct MLCL_Bits *mb = (struct MLCL_Bits *) _mb;
+
 	mlit = dmap_structure_add (mb->mlcl, DMAP_CC_MLIT);
+	g_object_get (record, "has-video", &has_video, NULL);
 
 	if (_dmap_share_client_requested (mb->bits, ITEM_KIND))
-		dmap_structure_add (mlit, DMAP_CC_MIKD, (gchar) DAAP_ITEM_KIND_AUDIO);
+		dmap_structure_add (mlit, DMAP_CC_MIKD,
+				    (gchar) DAAP_ITEM_KIND_AUDIO);
 	if (_dmap_share_client_requested (mb->bits, ITEM_ID))
-		dmap_structure_add (mlit, DMAP_CC_MIID, GPOINTER_TO_UINT (id));
+		dmap_structure_add (mlit, DMAP_CC_MIID,
+				    GPOINTER_TO_UINT (id));
 	if (_dmap_share_client_requested (mb->bits, ITEM_NAME)) {
 		gchar *title = NULL;
+
 		g_object_get (record, "title", &title, NULL);
 		if (title) {
 			dmap_structure_add (mlit, DMAP_CC_MINM, title);
 			g_free (title);
 		} else
-			g_warning ("Title requested but not available");
+			g_debug ("Title requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, PERSISTENT_ID))
-		dmap_structure_add (mlit, DMAP_CC_MPER, GPOINTER_TO_UINT (id));
+		dmap_structure_add (mlit, DMAP_CC_MPER,
+				    GPOINTER_TO_UINT (id));
 	if (_dmap_share_client_requested (mb->bits, CONTAINER_ITEM_ID))
-		dmap_structure_add (mlit, DMAP_CC_MCTI, GPOINTER_TO_UINT (id));
+		dmap_structure_add (mlit, DMAP_CC_MCTI,
+				    GPOINTER_TO_UINT (id));
 	if (_dmap_share_client_requested (mb->bits, SONG_DATA_KIND))
-		dmap_structure_add (mlit, DMAP_CC_ASDK, (gchar) DAAP_SONG_DATA_KIND_NONE);
+		dmap_structure_add (mlit, DMAP_CC_ASDK,
+				    (gchar) DAAP_SONG_DATA_KIND_NONE);
 	/* FIXME: Any use for this?
-	if (_dmap_share_client_requested (mb->bits, SONG_DATA_URL))
-		dmap_structure_add (mlit, DMAP_CC_ASUL, "daap://192.168.0.100:%u/databases/1/items/%d.%s?session-id=%s", data->port, *id, daap_record_get_format (DAAP_RECORD (record)), data->session_id);
-	*/
+	 * if (_dmap_share_client_requested (mb->bits, SONG_DATA_URL))
+	 * dmap_structure_add (mlit, DMAP_CC_ASUL, "daap://192.168.0.100:%u/databases/1/items/%d.%s?session-id=%s", data->port, *id, daap_record_get_format (DAAP_RECORD (record)), data->session_id);
+	 */
 	if (_dmap_share_client_requested (mb->bits, SONG_ALBUM)) {
 		gchar *album = NULL;
+
 		g_object_get (record, "songalbum", &album, NULL);
 		if (album) {
 			dmap_structure_add (mlit, DMAP_CC_ASAL, album);
 			g_free (album);
 		} else
-			g_warning ("Album requested but not available");
+			g_debug ("Album requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_GROUPING))
 		dmap_structure_add (mlit, DMAP_CC_AGRP, "");
 	if (_dmap_share_client_requested (mb->bits, SONG_ARTIST)) {
 		gchar *artist = NULL;
+
 		g_object_get (record, "songartist", &artist, NULL);
 		if (artist) {
 			dmap_structure_add (mlit, DMAP_CC_ASAR, artist);
 			g_free (artist);
 		} else
-			g_warning ("Artist requested but not available");
+			g_debug ("Artist requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_BITRATE)) {
 		gint32 bitrate = 0;
+
 		g_object_get (record, "bitrate", &bitrate, NULL);
 		if (bitrate != 0)
-			dmap_structure_add (mlit, DMAP_CC_ASBR, (gint32) bitrate);
+			dmap_structure_add (mlit, DMAP_CC_ASBR,
+					    (gint32) bitrate);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_BPM))
 		dmap_structure_add (mlit, DMAP_CC_ASBT, (gint32) 0);
@@ -610,11 +591,13 @@ add_entry_to_mlcl (gpointer id,
 		dmap_structure_add (mlit, DMAP_CC_ASCP, "");
 	if (_dmap_share_client_requested (mb->bits, SONG_DATE_ADDED)) {
 		gint32 firstseen = 0;
+
 		g_object_get (record, "firstseen", &firstseen, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASDA, firstseen);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_DATE_MODIFIED)) {
 		gint32 mtime = 0;
+
 		g_object_get (record, "mtime", &mtime, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASDM, mtime);
 	}
@@ -622,6 +605,7 @@ add_entry_to_mlcl (gpointer id,
 		dmap_structure_add (mlit, DMAP_CC_ASDC, (gint32) 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_DISC_NUMBER)) {
 		gint32 disc = 0;
+
 		g_object_get (record, "disc", &disc, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASDN, disc);
 	}
@@ -631,33 +615,43 @@ add_entry_to_mlcl (gpointer id,
 		dmap_structure_add (mlit, DMAP_CC_ASEQ, "");
 	if (_dmap_share_client_requested (mb->bits, SONG_FORMAT)) {
 		gchar *format = NULL;
-		if (transcode_format)
-			format = g_strdup (transcode_format);
-		else
+		gchar *transcode_mimetype = NULL;
+
+		g_object_get (mb->share, "transcode-mimetype",
+			      &transcode_mimetype, NULL);
+		// Not presently transcoding videos (see also same comments elsewhere).
+		if (! has_video && transcode_mimetype) {
+			format = g_strdup (mime_to_format
+					   (transcode_mimetype));
+			g_free (transcode_mimetype);
+		} else {
 			g_object_get (record, "format", &format, NULL);
+		}
 		if (format) {
 			dmap_structure_add (mlit, DMAP_CC_ASFM, format);
 			g_free (format);
 		} else
-			g_warning ("Format requested but not available");
+			g_debug ("Format requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_GENRE)) {
 		gchar *genre = NULL;
+
 		g_object_get (record, "songgenre", &genre, NULL);
 		if (genre) {
 			dmap_structure_add (mlit, DMAP_CC_ASGN, genre);
 			g_free (genre);
 		} else
-			g_warning ("Genre requested but not available");
+			g_debug ("Genre requested but not available");
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_DESCRIPTION))
-		dmap_structure_add (mlit, DMAP_CC_ASDT, ""); /* FIXME: e.g., wav audio file */
+		dmap_structure_add (mlit, DMAP_CC_ASDT, "");	/* FIXME: e.g., wav audio file */
 	if (_dmap_share_client_requested (mb->bits, SONG_RELATIVE_VOLUME))
 		dmap_structure_add (mlit, DMAP_CC_ASRV, 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_SAMPLE_RATE))
 		dmap_structure_add (mlit, DMAP_CC_ASSR, 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_SIZE)) {
 		guint64 filesize = 0;
+
 		g_object_get (record, "filesize", &filesize, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASSZ, (gint32) filesize);
 	}
@@ -667,6 +661,7 @@ add_entry_to_mlcl (gpointer id,
 		dmap_structure_add (mlit, DMAP_CC_ASSP, 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_TIME)) {
 		gint32 duration;
+
 		g_object_get (record, "duration", &duration, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASTM, (1000 * duration));
 	}
@@ -674,77 +669,88 @@ add_entry_to_mlcl (gpointer id,
 		dmap_structure_add (mlit, DMAP_CC_ASTC, 0);
 	if (_dmap_share_client_requested (mb->bits, SONG_TRACK_NUMBER)) {
 		gint32 track = 0;
+
 		g_object_get (record, "track", &track, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASTN, track);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_USER_RATING)) {
 		gint32 rating = 0;
+
 		g_object_get (record, "rating", &rating, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASUR, rating);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_YEAR)) {
 		gint32 year = 0;
+
 		g_object_get (record, "year", &year, NULL);
 		dmap_structure_add (mlit, DMAP_CC_ASYR, year);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_HAS_VIDEO)) {
-		gboolean has_video = 0;
-		g_object_get (record, "has-video", &has_video, NULL);
 		dmap_structure_add (mlit, DMAP_CC_AEHV, has_video);
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_SORT_ARTIST)) {
 		gchar *sort_artist = NULL;
+
 		g_object_get (record, "sort-artist", &sort_artist, NULL);
-		if (sort_artist)
+		if (sort_artist) {
 			dmap_structure_add (mlit, DMAP_CC_ASSA, sort_artist);
-		else
-			g_warning ("Sort artist requested but not available");
+			g_free (sort_artist);
+		} else {
+			g_debug ("Sort artist requested but not available");
+		}
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_SORT_ALBUM)) {
 		gchar *sort_album = NULL;
+
 		g_object_get (record, "sort-album", &sort_album, NULL);
-		if (sort_album)
+		if (sort_album) {
 			dmap_structure_add (mlit, DMAP_CC_ASSU, sort_album);
-		else
-			g_warning ("Sort album requested but not available");
+			g_free (sort_album);
+		} else {
+			g_debug ("Sort album requested but not available");
+		}
 	}
 	if (_dmap_share_client_requested (mb->bits, SONG_MEDIAKIND)) {
 		gint mediakind = 0;
+
 		g_object_get (record, "mediakind", &mediakind, NULL);
 		dmap_structure_add (mlit, DMAP_CC_AEMK, mediakind);
 	}
 }
 
 static void
-genre_tabulator (gpointer id, DMAPRecord *record, GHashTable *ht)
+genre_tabulator (gpointer id, DMAPRecord * record, GHashTable * ht)
 {
 	const gchar *genre;
+
 	g_object_get (record, "songgenre", &genre, NULL);
 	if (!genre)
 		return;
-	if (! g_hash_table_lookup (ht, genre))
+	if (!g_hash_table_lookup (ht, genre))
 		g_hash_table_insert (ht, (gchar *) genre, NULL);
 }
 
 static void
-artist_tabulator (gpointer id, DMAPRecord *record, GHashTable *ht)
+artist_tabulator (gpointer id, DMAPRecord * record, GHashTable * ht)
 {
 	const gchar *artist;
+
 	g_object_get (record, "songartist", &artist, NULL);
 	if (!artist)
 		return;
-	if (! g_hash_table_lookup (ht, artist))
+	if (!g_hash_table_lookup (ht, artist))
 		g_hash_table_insert (ht, (gchar *) artist, NULL);
 }
 
 static void
-album_tabulator (gpointer id, DMAPRecord *record, GHashTable *ht)
+album_tabulator (gpointer id, DMAPRecord * record, GHashTable * ht)
 {
 	const gchar *album;
+
 	g_object_get (record, "songalbum", &album, NULL);
 	if (!album)
 		return;
-	if (! g_hash_table_lookup (ht, album))
+	if (!g_hash_table_lookup (ht, album))
 		g_hash_table_insert (ht, (gchar *) album, NULL);
 }
 
@@ -759,22 +765,21 @@ add_to_category_listing (gpointer key, gpointer user_data)
 }
 
 static void
-databases_browse_xxx (DMAPShare *share,
-		      SoupServer *server,
-		      SoupMessage *msg,
+databases_browse_xxx (DMAPShare * share,
+		      SoupServer * server,
+		      SoupMessage * msg,
 		      const char *path,
-		      GHashTable *query,
-		      SoupClientContext *context)
+		      GHashTable * query, SoupClientContext * context)
 {
 	/* ABRO database browse
-         *      MSTT status
-         *      MUTY update type
-         *      MTCO specified total count
-         *      MRCO returned count
-         *      ABGN genre listing
-         *              MLIT listing item
-         *              ...
-         */
+	 *      MSTT status
+	 *      MUTY update type
+	 *      MTCO specified total count
+	 *      MRCO returned count
+	 *      ABGN genre listing
+	 *              MLIT listing item
+	 *              ...
+	 */
 	DMAPDb *db;
 	const gchar *rest_of_path;
 	GNode *abro, *node;
@@ -797,13 +802,16 @@ databases_browse_xxx (DMAPShare *share,
 	filtered = dmap_db_apply_filter (db, filter_def);
 
 	if (g_ascii_strcasecmp (browse_category, "genres") == 0) {
-		g_hash_table_foreach (filtered, (GHFunc) genre_tabulator, category_items);
+		g_hash_table_foreach (filtered, (GHFunc) genre_tabulator,
+				      category_items);
 		category_cc = DMAP_CC_ABGN;
 	} else if (g_ascii_strcasecmp (browse_category, "artists") == 0) {
-		g_hash_table_foreach (filtered, (GHFunc) artist_tabulator, category_items);
+		g_hash_table_foreach (filtered, (GHFunc) artist_tabulator,
+				      category_items);
 		category_cc = DMAP_CC_ABAR;
 	} else if (g_ascii_strcasecmp (browse_category, "albums") == 0) {
-		g_hash_table_foreach (filtered, (GHFunc) album_tabulator, category_items);
+		g_hash_table_foreach (filtered, (GHFunc) album_tabulator,
+				      category_items);
 		category_cc = DMAP_CC_ABAL;
 	} else {
 		g_warning ("Unsupported browse category: %s",
@@ -824,16 +832,17 @@ databases_browse_xxx (DMAPShare *share,
 	values = g_hash_table_get_keys (category_items);
 	if (values && g_hash_table_lookup (query, "include-sort-headers")) {
 		g_debug ("Sorting...");
-		values = g_list_sort (values, (GCompareFunc) g_ascii_strcasecmp);
+		values = g_list_sort (values,
+				      (GCompareFunc) g_ascii_strcasecmp);
 	}
 
 	g_list_foreach (values, add_to_category_listing, node);
-	
+
 	g_list_free (values);
 
 	_dmap_share_message_set_from_dmap_structure (share, msg, abro);
 	dmap_structure_destroy (abro);
-_bad_category:
+      _bad_category:
 	dmap_share_free_filter (filter_def);
 	/* Free's hash table but not data (points into real DB): */
 	g_hash_table_destroy (filtered);
@@ -841,12 +850,11 @@ _bad_category:
 }
 
 static void
-databases_items_xxx (DMAPShare *share,
-		     SoupServer *server,
-		     SoupMessage *msg,
+databases_items_xxx (DMAPShare * share,
+		     SoupServer * server,
+		     SoupMessage * msg,
 		     const char *path,
-		     GHashTable *query,
-		     SoupClientContext *context)
+		     GHashTable * query, SoupClientContext * context)
 {
 	DMAPDb *db;
 	const gchar *transcode_mimetype;
@@ -867,19 +875,26 @@ databases_items_xxx (DMAPShare *share,
 	g_object_get (record, "filesize", &filesize, NULL);
 
 	DMAP_SHARE_GET_CLASS (share)->message_add_standard_headers
-				(share, msg);
-	soup_message_headers_append (msg->response_headers, "Accept-Ranges", "bytes");
+		(share, msg);
+	soup_message_headers_append (msg->response_headers, "Accept-Ranges",
+				     "bytes");
 
-	range_header = soup_message_headers_get (msg->request_headers, "Range");
+	range_header =
+		soup_message_headers_get (msg->request_headers, "Range");
 	if (range_header) {
 		const gchar *s;
 		gchar *content_range;
 
-		s = range_header + 6; /* bytes= */
+		s = range_header + 6;	/* bytes= */
 		offset = atoll (s);
 
-		content_range = g_strdup_printf ("bytes %" G_GUINT64_FORMAT "-%" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT, offset, filesize, filesize);
-		soup_message_headers_append (msg->response_headers, "Content-Range", content_range);
+		content_range =
+			g_strdup_printf ("bytes %" G_GUINT64_FORMAT "-%"
+					 G_GUINT64_FORMAT "/%"
+					 G_GUINT64_FORMAT, offset, filesize,
+					 filesize);
+		soup_message_headers_append (msg->response_headers,
+					     "Content-Range", content_range);
 		g_debug ("Content range is %s.", content_range);
 		g_free (content_range);
 		soup_message_set_status (msg, SOUP_STATUS_PARTIAL_CONTENT);
@@ -887,13 +902,14 @@ databases_items_xxx (DMAPShare *share,
 		soup_message_set_status (msg, SOUP_STATUS_OK);
 	}
 	g_object_get (share, "transcode-mimetype", &transcode_mimetype, NULL);
-	send_chunked_file (server, msg, record, filesize, offset, transcode_mimetype);
-	
+	send_chunked_file (server, msg, record, filesize, offset,
+			   transcode_mimetype);
+
 	g_object_unref (record);
 }
 
 static struct DMAPMetaDataMap *
-get_meta_data_map (DMAPShare *share)
+get_meta_data_map (DMAPShare * share)
 {
 	return meta_data_map;
 }
