@@ -1,19 +1,26 @@
 #ifndef __DMAP_H
 #define __DMAP_H
 
+#include <libdmapsharing/daap-connection.h>
 #include <libdmapsharing/daap-record.h>
 #include <libdmapsharing/daap-share.h>
 #include <libdmapsharing/dmap-connection.h>
 #include <libdmapsharing/dmap-container-db.h>
 #include <libdmapsharing/dmap-container-record.h>
 #include <libdmapsharing/dmap-db.h>
+#include <libdmapsharing/dmap-enums.h>
+#include <libdmapsharing/dmap-gst-input-stream.h>
 #include <libdmapsharing/dmap-md5.h>
 #include <libdmapsharing/dmap-mdns-browser.h>
 #include <libdmapsharing/dmap-mdns-publisher.h>
 #include <libdmapsharing/dmap-record.h>
 #include <libdmapsharing/dmap-record-factory.h>
 #include <libdmapsharing/dmap-share.h>
+#include <libdmapsharing/dpap-connection.h>
 #include <libdmapsharing/dpap-record.h>
 #include <libdmapsharing/dpap-share.h>
+#include <libdmapsharing/dacp-share.h>
+#include <libdmapsharing/dacp-player.h>
+#include <libdmapsharing/dacp-connection.h>
 
 #endif /* __DMAP_H */
