@@ -20,6 +20,7 @@
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <libdmapsharing/dmap.h>
 
@@ -46,6 +47,9 @@ print_record (gpointer id, DMAPRecord *record, gpointer user_data)
 		     NULL);
 
 	g_print ("%d: %s %s\n", GPOINTER_TO_UINT (id), artist, title);
+
+	g_free (artist);
+	g_free (title);
 }
 
 static void
@@ -57,8 +61,6 @@ connected_cb (DMAPConnection *connection,
 	g_print ("Connection cb., DB has %lu entries\n", dmap_db_count (db));
 
 	dmap_db_foreach (db, (GHFunc) print_record, NULL);
-
-	g_main_loop_quit (loop);
 }
 
 static void
@@ -89,7 +91,7 @@ service_added_cb (DMAPMdnsBrowser *browser,
     DMAPConnection *conn;
     DMAPDb *db;
 
-    g_debug ("service added %s:%s:%s:%d (%s)",
+    g_debug ("service added %s:%s:%s:%d",
              service->service_name,
              service->name,
              service->host,
